@@ -22,7 +22,7 @@
 #include <string.h>
 #include "misc.h"
 
-extern panel *p;
+extern Panel *p;	/* FIXME: this should be removed */
 
 static gboolean setup_auto_complete( gpointer entry )
 {
@@ -119,7 +119,7 @@ void gtk_run()
     entry = gtk_entry_new();
 
     /* fix background */
-    gtk_widget_set_style(win, p->defstyle);
+    //gtk_widget_set_style(win, p->defstyle);
 
     gtk_entry_set_activates_default( (GtkEntry*)entry, TRUE );
     gtk_box_pack_start( (GtkBox*)((GtkDialog*)win)->vbox,
