@@ -24,10 +24,8 @@
 #include "panel.h"
 #include "misc.h"
 #include "plugin.h"
-//#define DEBUG
 #include "dbg.h"
 
-
 typedef struct {
     GtkTooltips *tips;
     char* image;
@@ -43,15 +41,13 @@ static pair wincmd_pair [] = {
     { 0, NULL },
 };
 
-
-
 static void
 toggle_shaded(wincmd *wc, guint32 action)
 {
     Window *win = NULL;
     int num, i;
     guint32 tmp2, dno;
-    net_wm_window_type nwwt;
+    NetWMWindowType nwwt;
 
     ENTER;
     win = get_xaproperty (GDK_ROOT_WINDOW(), a_NET_CLIENT_LIST, XA_WINDOW, &num);
@@ -98,7 +94,7 @@ toggle_iconify(wincmd *wc, guint32 action)
     Window *win = NULL;
     int num, i;
     guint32 tmp2, dno;
-    net_wm_window_type nwwt;
+    NetWMWindowType nwwt;
 
     ENTER;
     win = get_xaproperty (GDK_ROOT_WINDOW(), a_NET_CLIENT_LIST, XA_WINDOW, &num);
@@ -138,14 +134,16 @@ toggle_iconify(wincmd *wc, guint32 action)
 }
 
 static gint
-clicked (GtkWidget *widget, GdkEventButton *event, gpointer data)
+clicked (GtkWidget *widget, GdkEventButton *event, Plugin* plugin)
 {
-    wincmd *wc = (wincmd *) data;
+    wincmd *wc = (wincmd *)plugin->priv;
 
     ENTER;
     if (event->type != GDK_BUTTON_PRESS)
         RET(FALSE);
 
+    ENTER2;
+
     if (event->button == 1) {
         wc->action1 = 1 - wc->action1;
         toggle_iconify(wc, wc->action1);
@@ -154,15 +152,16 @@ clicked (GtkWidget *widget, GdkEventButton *event, gpointer data)
         wc->action2 = 1 - wc->action2;
         toggle_shaded(wc, wc->action2);
         DBG("wincmd: shade all\n");
-    } else {
-        DBG("wincmd: unsupported command\n");
+    } else if( event->button == 3 ) { /* right button */
+        GtkMenu* popup = lxpanel_get_panel_menu( plugin->panel, plugin, FALSE );
+        gtk_menu_popup( popup, NULL, NULL, NULL, NULL, event->button, event->time );
+        return TRUE;
     }
-
     RET(FALSE);
 }
 
 static void
-wincmd_destructor(plugin *p)
+wincmd_destructor(Plugin *p)
 {
     wincmd *wc = (wincmd *)p->priv;
 
@@ -176,7 +175,7 @@ wincmd_destructor(plugin *p)
 
 
 static int
-wincmd_constructor(plugin *p, char **fp)
+wincmd_constructor(Plugin *p, char **fp)
 {
     line s;
     gchar *fname;
@@ -248,7 +247,7 @@ wincmd_constructor(plugin *p, char **fp)
     button = fb_button_new_from_file(fname, w, h, 0x202020, TRUE);
     gtk_container_set_border_width(GTK_CONTAINER(button), 0);
     g_signal_connect(G_OBJECT(button), "button_press_event",
-          G_CALLBACK(clicked), (gpointer)wc);
+          G_CALLBACK(clicked), (gpointer)p);
 
     gtk_widget_show(button);
 
@@ -267,7 +266,7 @@ wincmd_constructor(plugin *p, char **fp)
     RET(0);
 }
 
-static void save_config( plugin* p, FILE* fp )
+static void save_config( Plugin* p, FILE* fp )
 {
     wincmd* wc = (wincmd*)p->priv;
 
@@ -277,7 +276,7 @@ static void save_config( plugin* p, FILE* fp )
 }
 
 
-plugin_class wincmd_plugin_class = {
+PluginClass wincmd_plugin_class = {
     fname: NULL,
     count: 0,
 
